/*  Denys VK3ZYZ (with help from Google and others)
  CTCSS generator

  20210303  Use an analog in for CTCSS selection.
  10 most common tones only. And a 440Hz test tone.
*/

#define Version "CTCSS_20210304"

int     timer       =  100;       // sub tone setting. The higher the number, the slower the timing.
int                     i ;
int     SineDivider = 4065;       // default sine wave divider
int     CTCSS       =    0;
int     freq        =    0;       // CTCSS index


#define On  1
#define Off 0
#define RefFrequ      500000      // 16Mhz / 32 steps for CTCSS sine wave generator
#define TRXmode       LOW         // Hi = RX, LOW =  TX
#define TRX           2           // input HIGH = RX, LOW = TX.
#define SelectTone    A7          // CTCSS_select1
#define CTCSSLED      13          // internal LED

/* ********************* PortB0-PB4 used to generate CTCSS tones via R2R nerwork.
#define spare1         8           // Sub Tone R2R b0 PB0
#define spare2         9           // Sub Tone R2R b1 PB1
#define spare3        10           // Sub Tone R2R b2 PB2
#define spare4        11           // Sub Tone R2R b3 PB3
#define spare5        12           // Sub Tone R2R b4 PB4
*/

float SubTone [] // Edit this for CTCSS frequencies wanted. Selected by SelectTone settings.
{   0.0,  //  0
   67.0, 
   74.4,  
   88.5,
   91.5,  
   94.8,
  103.5,
  118.8,
  123.0,
  141.3,
  179.9, 
  440.0,  // test tone
};

int sinetable[] // one cycle sine wave via R2R network.
{ 0,1,2,3,5,7,10,13,16,19,22,25,27,28,29,30,31,30,29,28,27,25,22,19,16,13,10,7,5,3,2,1, };


void setup() {
  Serial.begin(9600);  
  pinMode(TRX,INPUT_PULLUP);      // Hi = RX, LOW =  TX
  pinMode(CTCSSLED,OUTPUT);
  digitalWrite(CTCSSLED,LOW);
  DDRB  = B011111;                // sets Arduino pins 8 to 12 as outputs.
  PORTB = B010000;                // R2R out at rest to half level.
  Serial.println (Version);

}

/******************************
  main loop
*******************************/
void loop() {
  int newRead = 0;                  // read SelectTone until twice the same....
  int oldRead = 0;
                                            // What CTCSS frequencty is selected?
  newRead=(analogRead(SelectTone));         // preset the reading
        while (newRead != oldRead)          // was it the same as last time?
        { oldRead = newRead;                // no, save the newest reading
        newRead=(analogRead(SelectTone));   // then read again. Max = 1024.
        }
  freq = newRead/90;                        // freq = 0 to 11 
  freq = constrain(freq, 0,11);

  if (digitalRead(TRX) ==TRXmode) 
  {                                 // Transmitting......
    // debug info
              Serial.print("ADC=");
              Serial.print(newRead);
              Serial.print("  Index=");
              Serial.print(freq);
              Serial.print("   CTCSS=");
              Serial.println((SubTone[freq]));
              
      SineDivider = 0.0;            // preset for zero selection.
      if ((SubTone[freq]) != 0)     // if the CTCSS frequ in not zero..
      { SineDivider = (RefFrequ/(SubTone[freq])); // get the CTCSS frequ needed.
      CTCSS_Tone();                 // Turn CTCSS on for this channel.
      digitalWrite(CTCSSLED,HIGH);
      }
      else 
      {
      TIMSK1 = 0;                   // Turn CTCSS off for this channel.
      digitalWrite(CTCSSLED,LOW);
      } 
   }
   while (digitalRead(TRX) ==TRXmode) 
   {                                // loop until back to RX mode
   }
   TIMSK1 = 0;                      // Turn CTCSS off for receive.  
   digitalWrite(CTCSSLED,LOW); 
   delay(100);
}

void CTCSS_Tone() // This is the Timer 1 interupt routine to generate the tone.
{
  cli();                    // interupts off for setup
  TCCR1A = 0;               // 
  TCCR1B = 0;               //  no prescaler
                            // set compare match register for 1hz increments
  OCR1A = SineDivider ;     // = (16*10^6) / (1*1024) - 1 (must be <65536)                       
  TCCR1B |= (1 << WGM12);   // turn on CTC mode                         
  TCCR1B |= (0 << CS12) | (0 << CS11) | (1 << CS10); // Set CS10 no prescaler                         
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt
  sei();                    // Interupts on.       
}

ISR(TIMER1_COMPA_vect)      // Each time the Timer 1 overflows,
{   PORTB=(sinetable[i++]); // write the next value from table to PORTB
    if(i>=32)               // Has the end of the sinewave table been reached?
    { i=0;                  // Yes, loop back to the start of the table.
    }
}
